@@ -268,6 +268,7 @@ fib(5)")
 
 (defvar minuet-claude-options
   `(:model "claude-3-5-haiku-20241022"
+    :end-point "https://api.anthropic.com/v1/messages"
     :max_tokens 512
     :api-key "ANTHROPIC_API_KEY"
     :system
@@ -285,7 +286,7 @@ fib(5)")
   "Config options for Minuet Claude provider.")
 
 (defvar minuet-openai-options
-  `(:model "gpt-4o-mini"
+  `(:model "gpt-4.1-mini"
     :api-key "OPENAI_API_KEY"
     :system
     (:template minuet-default-system-template
@@ -317,7 +318,7 @@ fib(5)")
 (defvar minuet-openai-compatible-options
   `(:end-point "https://openrouter.ai/api/v1/chat/completions"
     :api-key "OPENROUTER_API_KEY"
-    :model "qwen/qwen2.5-32b-instruct"
+    :model "mistralai/devstral-small-2505"
     :system
     (:template minuet-default-system-template
      :prompt minuet-default-prompt
@@ -348,6 +349,7 @@ fib(5)")
 
 (defvar minuet-gemini-options
   `(:model "gemini-2.0-flash"
+    :end-point "https://generativelanguage.googleapis.com/v1beta/models"
     :api-key "GEMINI_API_KEY"
     :system
     (:template minuet-default-system-template
@@ -370,17 +372,37 @@ fib(5)")
        (not (or (evil-insert-state-p)
                 (evil-emacs-state-p)))))
 
-(defun minuet-set-optional-options (options key val &optional field)
-  "Set the value of KEY in the FIELD of OPTIONS to VAL.
-If FIELD is not provided, it defaults to :optional.  If VAL is nil,
+(defmacro minuet-set-nested-plist (plist val &rest attributes)
+  "Set or delete a PLIST's nested ATTRIBUTES.
+PLIST is the plist to set.  If VAL is non-nil, set the nested
+attribute to VAL.  If VAL is nil, delete the final attribute from its
+parent plist.
+Example usage:
+\(minuet-set-nested-plist `minuet-openai-options' 256 :optional :max-tokens)
+;; delete :max-tokens field
+\(minuet-set-nested-plist `minuet-openai-options' nil :optional :max-tokens)"
+  (if (null attributes)
+      (error "Minuet-set-nested-plist requires at least one attribute key"))
+  (if val
+      (let ((access-form plist))
+        (dolist (attr attributes)
+          (setq access-form `(plist-get ,access-form ,attr)))
+        `(setf ,access-form ,val))
+    ;; If val is nil, delete the key from its parent plist.
+    (let* ((all-but-last-attributes (butlast attributes))
+           (last-attribute (car (last attributes)))
+           (parent-plist-accessor plist))
+      (dolist (attr all-but-last-attributes)
+        (setq parent-plist-accessor `(plist-get ,parent-plist-accessor ,attr)))
+      `(setf ,parent-plist-accessor (map-delete ,parent-plist-accessor ,last-attribute)))))
+
+(defun minuet-set-optional-options (options key val &optional parent-key)
+  "Set the value of KEY in the PARENT-KEY of OPTIONS to VAL.
+If PARENT-KEY is not provided, it defaults to :optional.  If VAL is nil,
 then remove KEY from OPTIONS.  This helper function simplifies setting
 values in a two-level nested plist structure."
-  (let ((field (or field :optional)))
-    (if val
-        (setf (plist-get options field)
-              (plist-put (plist-get options field) key val))
-      (setf (plist-get options field)
-            (map-delete (plist-get options field) key)))))
+  (let ((parent-key (or parent-key :optional)))
+    (minuet-set-nested-plist options val parent-key key)))
 
 (defun minuet--eval-value (value)
   "Eval a VALUE for minuet.
@@ -396,8 +418,8 @@ symbol, return its value.  Else return itself."
   (when minuet--current-requests
     (dolist (proc minuet--current-requests)
       (when (process-live-p proc)
-        (minuet--log (format "%s process killed" (prin1-to-string proc)))
-        (delete-process proc)))
+        (minuet--log (format "%s process terminated" (prin1-to-string proc)))
+        (signal-process proc 'SIGTERM)))
     (setq minuet--current-requests nil)))
 
 (defun minuet--cleanup-suggestion (&optional no-cancel)
@@ -507,7 +529,7 @@ Also cancel any pending requests unless NO-CANCEL is t."
     (funcall complete-fn
              context
              (lambda (items)
-               (setq items (-distinct items))
+               (setq items (seq-uniq items))
                (with-current-buffer current-buffer
                  (when (and items (not (minuet--cursor-moved-p)))
                    (minuet--display-suggestion items 0)))))))
@@ -603,34 +625,42 @@ Also print the MESSAGE when MESSAGE-P is t."
 
 (defun minuet--make-chat-llm-shot (context options)
   "Build the final chat input for chat llm.
-CONTEXT is read from current buffer content.
-OPTIONS should be the provider options plist."
+The CONTEXT is read from the current buffer content.  OPTIONS should
+be specified as a plist of provider settings.  The return value will a
+list of strings.  It will then be converted into a multi-turn
+conversation with alternating `user` and `assistant` roles by
+`minuet--create-chat-messages-from-list'"
   (let* ((chat-input (copy-tree (plist-get options :chat-input)))
-         (template (minuet--eval-value (plist-get chat-input :template)))
-         (parts nil))
+         (templates (minuet--eval-value (plist-get chat-input :template)))
+         (templates (if (stringp templates) (list templates) templates))
+         (parts nil)
+         (results nil))
     ;; Remove template from options to avoid infinite recursion
     (setq chat-input (plist-put chat-input :template nil))
     ;; Use cl-loop for better control flow
-    (cl-loop with last-pos = 0
-             for match = (string-match "{{{\\(.+?\\)}}}" template last-pos)
-             until (not match)
-             for start-pos = (match-beginning 0)
-             for end-pos = (match-end 0)
-             for key = (match-string 1 template)
-             do
-             ;; Add text before placeholder
-             (when (> start-pos last-pos)
-               (push (substring template last-pos start-pos) parts))
-             ;; Get and add replacement value
-             (when-let* ((repl-fn (plist-get chat-input (intern key)))
-                         (value (funcall repl-fn context)))
-               (push value parts))
-             (setq last-pos end-pos)
-             finally
-             ;; Add remaining text after last match
-             (push (substring template last-pos) parts))
-    ;; Join parts in reverse order
-    (apply #'concat (nreverse parts))))
+    (dolist (template templates)
+      (setq parts nil)
+      (cl-loop with last-pos = 0
+               for match = (string-match "{{{\\(.+?\\)}}}" template last-pos)
+               until (not match)
+               for start-pos = (match-beginning 0)
+               for end-pos = (match-end 0)
+               for key = (match-string 1 template)
+               do
+               ;; Add text before placeholder
+               (when (> start-pos last-pos)
+                 (push (substring template last-pos start-pos) parts))
+               ;; Get and add replacement value
+               (when-let* ((repl-fn (plist-get chat-input (intern key)))
+                           (value (funcall repl-fn context)))
+                 (push value parts))
+               (setq last-pos end-pos)
+               finally
+               ;; Add remaining text after last match
+               (push (substring template last-pos) parts))
+      ;; Join parts in reverse order
+      (push (apply #'concat (nreverse parts)) results))
+    (nreverse results)))
 
 (defun minuet--make-context-filter-sequence (context len)
   "Create a filtering string based on CONTEXT with maximum length LEN."
@@ -797,7 +827,7 @@ many lines.  Without a prefix argument, accept only the first line."
                  (setq items (if minuet-add-single-line-entry
                                  (minuet--add-single-line-entry items)
                                items)
-                       items (-distinct items))
+                       items (seq-uniq items))
                  ;; close current minibuffer session, if any
                  (when (active-minibuffer-window)
                    (abort-recursive-edit))
@@ -993,6 +1023,19 @@ CONTEXT and CALLBACK will be passed to the base function."
        (plist-get it :delta)
        (plist-get it :content)))
 
+(defun minuet--create-chat-messages-from-list (str-list)
+  "Convert a list of strings into alternating user/assistant chat messages.
+STR-LIST is a list of strings.  Returns a list of plists with :role
+and :content keys."
+  (let ((result nil)
+        (roles '("user" "assistant")))
+    (cl-loop for i from 1 to (length str-list)
+             for content in str-list
+             do (push (list :role (nth (mod (1- i) 2) roles)
+                            :content content)
+                      result))
+    (nreverse result)))
+
 (defun minuet--openai-complete-base (options context callback)
   "The base function to complete code with openai API.
 OPTIONS are the provider options.  the completion items from json.
@@ -1013,10 +1056,10 @@ to be called when completion items arrive."
           :model ,(plist-get options :model)
           :messages ,(vconcat
                       `((:role "system"
-                         :content ,(minuet--make-system-prompt (plist-get options :system)))
-                        ,@(minuet--eval-value (plist-get options :fewshots))
-                        (:role "user"
-                         :content ,(minuet--make-chat-llm-shot context options))))))
+                         :content ,(minuet--make-system-prompt (plist-get options :system))))
+                      (minuet--eval-value (plist-get options :fewshots))
+                      (--> (minuet--make-chat-llm-shot context options)
+                           minuet--create-chat-messages-from-list))))
        :as 'string
        :filter (minuet--make-process-stream-filter --response--)
        :then
@@ -1061,7 +1104,7 @@ CONTEXT is to be used to build the prompt.  CALLBACK is the function
 to be called when completion items arrive."
   (minuet--with-temp-response
     (push
-     (plz 'post "https://api.anthropic.com/v1/messages"
+     (plz 'post (plist-get minuet-claude-options :end-point)
        :headers `(("Content-Type" . "application/json")
                   ("Accept" . "application/json")
                   ("x-api-key" . ,(minuet--get-api-key (plist-get minuet-claude-options :api-key)))
@@ -1076,9 +1119,9 @@ to be called when completion items arrive."
             :system ,(minuet--make-system-prompt (plist-get options :system))
             :max_tokens ,(plist-get options :max_tokens)
             :messages ,(vconcat
-                        `(,@(minuet--eval-value (plist-get options :fewshots))
-                          (:role "user"
-                           :content ,(minuet--make-chat-llm-shot context minuet-claude-options)))))))
+                        (minuet--eval-value (plist-get options :fewshots))
+                        (--> (minuet--make-chat-llm-shot context options)
+                             minuet--create-chat-messages-from-list)))))
        :as 'string
        :filter (minuet--make-process-stream-filter --response--)
        :then
@@ -1107,13 +1150,30 @@ to be called when completion items arrive."
        car
        (plist-get it :text)))
 
+(defun minuet--transform-openai-chat-to-gemini-chat (chat)
+  "Convert OpenAI-format chat to Gemini format.
+CHAT is a list of plists with :role and :content keys"
+  (let (new-chat)
+    (dolist (message chat)
+      (let ((gemini-message
+             (pcase (plist-get message :role)
+               ("user"
+                `(:role "user" :parts [(:text ,(plist-get message :content))]))
+               ("assistant"
+                `(:role "model" :parts [(:text ,(plist-get message :content))]))
+               (_ nil))))
+        (when gemini-message
+          (push gemini-message new-chat))))
+    (nreverse new-chat)))
+
 (defun minuet--gemini-complete (context callback)
   "Complete code with gemini.
 CONTEXT is to be used to build the prompt.  CALLBACK is the function
 to be called when completion items arrive."
   (minuet--with-temp-response
     (push
-     (plz 'post (format "https://generativelanguage.googleapis.com/v1beta/models/%s:streamGenerateContent?alt=sse&key=%s"
+     (plz 'post (format "%s/%s:streamGenerateContent?alt=sse&key=%s"
+                        (plist-get minuet-gemini-options :end-point)
                         (plist-get minuet-gemini-options :model)
                         (minuet--get-api-key (plist-get minuet-gemini-options :api-key)))
        :headers `(("Content-Type" . "application/json")
@@ -1123,19 +1183,14 @@ to be called when completion items arrive."
        (json-serialize
         (let* ((options (copy-tree minuet-gemini-options))
                (fewshots (minuet--eval-value (plist-get options :fewshots)))
-               (fewshots (mapcar
-                          (lambda (shot)
-                            `(:role
-                              ,(if (equal (plist-get shot :role) "user") "user" "model")
-                              :parts
-                              [(:text ,(plist-get shot :content))]))
-                          fewshots)))
+               (fewshots (minuet--transform-openai-chat-to-gemini-chat fewshots)))
           `(,@(plist-get options :optional)
             :system_instruction (:parts (:text ,(minuet--make-system-prompt (plist-get options :system))))
             :contents ,(vconcat
-                        `(,@fewshots
-                          (:role "user"
-                           :parts [(:text ,(minuet--make-chat-llm-shot context minuet-gemini-options))]))))))
+                        fewshots
+                        (--> (minuet--make-chat-llm-shot context options)
+                             minuet--create-chat-messages-from-list
+                             minuet--transform-openai-chat-to-gemini-chat)))))
        :as 'string
        :filter (minuet--make-process-stream-filter --response--)
        :then
